@@ -1,28 +1,23 @@
 #define _GNU_SOURCE
 
 #include "ctr_exit.h"
-#include "cli.h" // opt_exit_command, opt_exit_delay
+#include "cli.h" // opt_exit_command, opt_exit_delay, opt_socket_path, opt_cuuid
 #include "utils.h"
 #include "parent_pipe_fd.h"
 #include "globals.h"
 #include "ctr_logging.h"
+#include "close_fds.h"
 
 #include <errno.h>
 #include <glib.h>
 #include <glib-unix.h>
 #include <signal.h>
 #include <stdlib.h>
-#include <sys/prctl.h>
 #include <unistd.h>
 
 volatile pid_t container_pid = -1;
 volatile pid_t create_pid = -1;
 
-void on_sigchld(G_GNUC_UNUSED int signal)
-{
-	raise(SIGUSR1);
-}
-
 void on_sig_exit(int signal)
 {
 	if (container_pid > 0) {
@@ -82,9 +77,13 @@ gboolean check_child_processes_cb(gpointer user_data)
 	return G_SOURCE_REMOVE;
 }
 
-gboolean on_sigusr1_cb(gpointer user_data)
+gboolean on_signalfd_cb(gint fd, G_GNUC_UNUSED GIOCondition condition, gpointer user_data)
 {
 	struct pid_check_data *data = (struct pid_check_data *)user_data;
+
+	/* drop the signal from the signalfd */
+	drop_signal_event(fd);
+
 	check_child_processes(data->pid_to_handler, data->exit_status_cache);
 	return G_SOURCE_CONTINUE;
 }
@@ -133,32 +132,16 @@ void container_exit_cb(G_GNUC_UNUSED GPid pid, int status, G_GNUC_UNUSED gpointe
 	g_main_loop_quit(main_loop);
 }
 
-static void close_fd(int *fd)
-{
-	if (*fd >= 0)
-		close(*fd);
-	*fd = -1;
-}
-
 void do_exit_command()
 {
-	if (sync_pipe_fd > 0) {
-		close(sync_pipe_fd);
-		sync_pipe_fd = -1;
-	}
-
 	if (signal(SIGCHLD, SIG_DFL) == SIG_ERR) {
 		_pexit("Failed to reset signal for SIGCHLD");
 	}
 
-	close_logging_fds();
-	close_fd(&terminal_ctrl_fd);
-	close_fd(&winsz_fd_w);
-	close_fd(&winsz_fd_r);
-	close_fd(&mainfd_stdin);
-	close_fd(&attach_socket_fd);
-	close_fd(&console_socket_fd);
-	close_fd(&attach_pipe_fd);
+	/*
+	 * Close everything except stdin, stdout and stderr.
+	 */
+	close_all_fds_ge_than(3);
 
 	/*
 	 * We don't want the exit command to be reaped by the parent conmon
@@ -166,7 +149,7 @@ void do_exit_command()
 	 * Unfortunately, that also means that any new subchildren from
 	 * still running processes could also get lost
 	 */
-	if (prctl(PR_SET_CHILD_SUBREAPER, 0) != 0) {
+	if (set_subreaper(false) != 0) {
 		nwarn("Failed to disable self subreaper attribute - might wait for indirect children a long time");
 	}
 
@@ -232,6 +215,16 @@ void reap_children()
 		;
 }
 
+void cleanup_socket_dir_symlink()
+{
+	/* A symbolic link might be created at {opt_socket_path}/{opt_cuuid} if the container
+	   is created successfully.
+	   This function will take care of removing the link when the conmon process exits. */
+	char *base_path = g_build_filename(opt_socket_path, opt_cuuid, NULL);
+	if (unlink(base_path) == -1 && errno != ENOENT)
+		pexitf("Failed to remove existing symlink for the socket directory %s", base_path);
+}
+
 void handle_signal(G_GNUC_UNUSED const int signum)
 {
 	exit(EXIT_FAILURE);
